uniform sampler2D DiffuseMap;
uniform sampler2D NormalMap;
uniform sampler2D ParamMap;

uniform samplerCube BlurEnv;
uniform samplerCube ReflEnv;

uniform vec4 AthmosphereColor;
uniform vec4 SurfaceLightColor;

varying vec2 vTexCoord;
varying vec3 vNormal;
varying vec3 vTangent;
varying vec3 vBinormal;

varying vec4 vPosition;

float OrenNayar(vec3 		normal,
								vec3 		light,
								vec3 		view,
								float 	sigma)
{
	float NL = clamp(dot(normal, light), 0.0, 0.999);
	float NV = clamp(dot(normal, view), 0.0, 0.999);
	
	float theta_r = acos(NV);
	float theta_i = acos(NL);
	
	float alpha, beta;
	if (theta_r > theta_i)
	{
		alpha = theta_r;
		beta = theta_i;
	}
	else
	{
		alpha = theta_i;
		beta = theta_r;
	}
	
	vec3 VonN = view - normal * NV;
	vec3 LonN = light - normal * NL;
	
	float norm = length(VonN) * length(LonN);

	float cos_delta_phi;
	if (norm < 0.001)
	{
		cos_delta_phi = 1.0;			
	}
	else
	{
		cos_delta_phi = dot(VonN, LonN) / norm;
	}
	
	float sigma2 = sigma * sigma;
	
	float A = 1.0 - 0.5 * (sigma2 / (sigma2 + 0.33));
	float B = 0.45 * (sigma2 / (sigma2 + 0.09));
	beta = clamp(beta, 0.0, 1.5708);
	
	return NL * (A + B * cos_delta_phi * sin(alpha) * tan(beta));
}

void main(void)
{
	vec3 lightDir = normalize(gl_LightSource[0].spotDirection);
	vec3 lightCol = gl_LightSource[0].diffuse.rgb;
	vec3 viewDir = normalize(vec4(0.0, 0.0, 0.0, 1.0) - vPosition);
	
	vec3 finalTangent = normalize(vTangent);
	vec3 finalBinormal = normalize(vBinormal);
	
	vec3 baseNormal = normalize(vNormal);	
	vec3 cloudNormal = baseNormal;
	
	vec4 normalSample = texture2D(NormalMap, vTexCoord);
	vec4 diffuseSample = texture2D(DiffuseMap, vTexCoord);
	vec4 paramSample = texture2D(ParamMap, vTexCoord);	
	
	baseNormal += vTangent * (normalSample.g * 2.0 - 1.0);
	baseNormal += vBinormal * (normalSample.a * 2.0 - 1.0);
	
	cloudNormal += vTangent * (paramSample.r * 2.0 - 1.0);
	cloudNormal += vBinormal * (paramSample.g * 2.0 - 1.0);
	
	baseNormal = normalize(baseNormal);
	cloudNormal = normalize(cloudNormal);

	float incidence = max(dot(viewDir, baseNormal), 0.0);

	float kD = OrenNayar(baseNormal, lightDir, viewDir, normalSample.b) * (normalSample.b + 1.0);
	kD = max(kD, 0.0);

	vec3 half = normalize(lightDir + viewDir);
	
	float kS = max(dot(baseNormal, half), 0.0);
	kS = pow(kS, 50.0 * (incidence * 0.5 + 0.5)) * (incidence * 0.9 + 0.1) * normalSample.r;
	
  vec3 reflvec = mat3(gl_ModelViewMatrixInverse) * reflect(viewDir, baseNormal);

	vec3 baseColor;
	baseColor = diffuseSample.rgb * kD * lightCol;
	baseColor += kS * (1.0 - normalSample.b) * lightCol;
	baseColor += SurfaceLightColor.rgb * diffuseSample.a * smoothstep(0.66,1.0,(1.0 - kD));
	baseColor += textureCube(BlurEnv, reflvec) * normalSample.r;
	
	vec3 cloudColor = vec3(1.0, 1.0, 1.0);
	kD = max(dot(cloudNormal, lightDir), 0.0);
	cloudColor *= kD;

  gl_FragColor.rgb = baseColor * (1.0 - paramSample.a) + cloudColor * paramSample.a;
  gl_FragColor.rgb += AthmosphereColor * pow(1.0 - incidence, 3.0) * (dot(baseNormal, lightDir) + 1.0) * 0.5;
  gl_FragColor.rgb += gl_LightSource[1].diffuse.rgb * diffuseSample.rgb * (1.0 - incidence);

  gl_FragColor.a = 1.0;
}
